package it.inaf.oats.ia2.tapschemamanager.api;

import it.inaf.oats.ia2.tapschemamanager.api.contract.DatabaseType;
import it.inaf.oats.ia2.tapschemamanager.api.contract.TapSchema;
import it.inaf.oats.ia2.tapschemamanager.api.contract.TapSchemaEntity;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Prepares an {@code UPDATE } SQL query for a given {@link TapSchemaEntity} and
 * a given {@link TapSchema}.
 *
 * @author Sonia Zorba {@literal <zorba at oats.inaf.it>}
 */
public class UpdateQueryBuilder {

    private static final Logger log = LoggerFactory.getLogger(UpdateQueryBuilder.class);

    private final String query;
    private final List<EntityPropertyInfo> addedProperties;
    private final TapSchemaEntity tapSchemaEntity;

    protected UpdateQueryBuilder(DatabaseType dbType, TapSchema tapSchema, TapSchemaEntity tapSchemaEntity, String tapSchemaTableName, String whereCondition) {

        StringBuilder querySb = new StringBuilder("UPDATE ");
        querySb.append(TSMUtil.escapeName(tapSchema.getName(), dbType));
        querySb.append(".");
        querySb.append(TSMUtil.escapeName(tapSchemaTableName, dbType));
        querySb.append("\nSET");

        addedProperties = new ArrayList<>();
        this.tapSchemaEntity = tapSchemaEntity;

        boolean first = true;
        for (EntityPropertyInfo propertyInfo : EntityPropertyInfo.getEntityPropertiesInfo(tapSchemaTableName)) {
            if (propertyInfo.isUpdatable()
                    && (propertyInfo.acceptVersion(tapSchema.getVersion()))
                    && tapSchemaEntity.isChanged(propertyInfo.getPropertyKey())) {

                if (!first) {
                    querySb.append(",");
                }
                querySb.append(" ");
                querySb.append(propertyInfo.getPropertyKey());
                querySb.append(" = ?");
                addedProperties.add(propertyInfo);
                first = false;
            }
        }

        querySb.append("\nWHERE ");
        querySb.append(whereCondition);

        query = querySb.toString();
    }

    public String getQuery() {
        return query;
    }

    protected int addStatementValues(PreparedStatement statement) throws SQLException {
        int i = 1;
        for (EntityPropertyInfo property : addedProperties) {
            Object value = tapSchemaEntity.getValue(property.getPropertyKey(), property.getPropertyType());
            statement.setObject(i, value, property.getSqlType());
            log.debug("\t[{}] {}", property.getPropertyKey(), value);
            i++;
        }
        return i;
    }
}
