package it.inaf.oats.ia2.tapschemamanager.api.contract;

/**
 * Define the persistence status of a {@link ChildEntity}.
 *
 * @author Sonia Zorba {@literal <zorba at oats.inaf.it>}
 */
public enum Status {

    /**
     * Information about the entity has been loaded from the database (e.g. for
     * performance reason) but the entity hasn't been added to the TAP_SCHEMA.
     * Currently only {@code Column}s uses this state.
     */
    LOADED,
    /**
     * The entity was added to the TAP_SCHEMA and this fact is stored into the
     * database.
     */
    ADDED_PERSISTED,
    /**
     * The entity is marked as added, but it hasn't been saved to the database
     * yet.
     */
    ADDED_NOT_PERSISTED,
    /**
     * The entity is marked as removed but it is still written into the
     * database, so it will be removed from it only when the
     * {@link TapSchema#save()} method is called.
     */
    TO_REMOVE,
    /**
     * The entity is marked as removed but it has never been written into the
     * database.
     */
    REMOVED_NOT_PERSISTED
}
